@@ -21,21 +21,28 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include "c-tokens.h"
+#include "tokens.h"
 #include "report.h"
 
+lexer_t     ratslexer;
 Hash        database = (Hash)NULL;
+Hash	    defaultdb = (Hash)NULL;
 char *      current_file;
 stack_t *   current_frame = (stack_t *)NULL;
 
 static int                  depths[DEPTH_COUNT];
 static int                  ungotten_token  = -1;
 static int                  toctou_count = 0;
+static int                  last_text_line = 0;
+static ignore_t *           current_ignore = (ignore_t *)NULL;
 static toctou_t *           toctous = (toctou_t *)NULL;
 static accumulator_t *      accumulators = (accumulator_t *)NULL;
 
 static void analyze_variable(void);
 static void analyze_identifier(void);
+static void analyze_comment(void);
+static void analyze_python_backticks(void);
+static argument_t *get_argument(int number);
 
 static
 int accumulate_text(accumulator_t *acc, char *text)
@@ -74,7 +81,7 @@ void push_accumulator(char **text)
     acc->next    = accumulators;
     accumulators = acc;
 
-    accumulate_text(acc, yytext);
+    accumulate_text(acc, *ratslexer.yytext);
 }
 
 static
@@ -91,7 +98,7 @@ void pop_accumulator(void)
             accumulators->length = 0;
             accumulate_text(accumulators, *(acc->text));
         }
-        acc->length -= strlen(yytext);
+        acc->length -= strlen(*ratslexer.yytext);
         *(*acc->text + acc->length) = '\0';
     }
     free(acc);
@@ -124,10 +131,10 @@ int get_token(void)
     }
     else
     {
-        if (!(token = yylex()))
+        if (!(token = (ratslexer.yylex)() ))
             return 0;
         if (accumulators != (accumulator_t *)NULL)
-            accumulate_text(accumulators, yytext);
+            accumulate_text(accumulators, *ratslexer.yytext);
     }
 
     switch (token)
@@ -143,6 +150,8 @@ int get_token(void)
     return token;
 }
 
+
+   
 static
 void scan_tokens(processorfn_t fn, void *arg)
 {
@@ -150,6 +159,15 @@ void scan_tokens(processorfn_t fn, void *arg)
 
     while ((token = get_token()) != 0)
     {
+        if (token != COMMENT)
+        {
+            last_text_line = *ratslexer.lex_lineno;
+            if (current_ignore != (ignore_t *)NULL)
+            {
+                current_ignore->lineno = *ratslexer.lex_lineno;
+                current_ignore = (ignore_t *)NULL;
+            }
+        }
         if (fn != NULL)
             if (!fn(token, arg))
                 break;
@@ -163,10 +181,45 @@ void scan_tokens(processorfn_t fn, void *arg)
             case IDENTIFIER:
                 analyze_identifier();
                 break;
+
+            case COMMENT:
+                analyze_comment();
+                break;
+
+            case '`':
+                if (ratslexer.lang == LANG_PYTHON)  
+                {
+                  analyze_python_backticks();
+                }
+                break;
+ 
         }
     }
 }
 
+static
+int gobble_backtick(int token, void *arg)
+{
+
+    if (token == '`')
+      return 0;
+    return 1;
+}
+
+static void
+analyze_python_backticks(void)
+{
+  int myline = *ratslexer.lex_lineno;
+  /* we're just gobbling up whatever is in the backticks and
+     ignoring them for now.
+  */
+
+  scan_tokens(gobble_backtick, NULL);
+
+  log_pythonbacktick(myline, Medium);
+}
+
+
 static
 int check_buffer(int token, void *arg)
 {
@@ -178,7 +231,9 @@ int check_buffer(int token, void *arg)
     if (data->skip)
         return 1;
 
-    if (token == '[' && data->last_token == IDENTIFIER)
+    if (token == '=')
+        data->skip = 1;
+    else if (token == '[' && data->last_token == IDENTIFIER)
         data->depth = depths[DEPTH_BRACE];
     else if (data->last_token == '[')
     {
@@ -213,7 +268,7 @@ void analyze_variable(void)
     if (depths[DEPTH_PARENTHESIS] || depths[DEPTH_BRACKET])
         return;
 
-    data.lineno = lex_lineno;
+    data.lineno = *ratslexer.lex_lineno;
     data.last_token = CHAR;
     data.depth = depths[DEPTH_BRACE];
     data.skip = 0;
@@ -221,8 +276,9 @@ void analyze_variable(void)
 }
 
 static
-int check_format_string(char *fmt)
+int check_format_string(char *fmt, int format_arg)
 {
+    int     arg = format_arg + 1;
     char *  c;
 
     for (c = strchr(fmt, '%');  c != (char *)NULL;  c = strchr((fmt = c + 1), '%'))
@@ -246,7 +302,10 @@ int check_format_string(char *fmt)
         }
 
         if (*c == '*')
+        {
+            arg++;
             c++;
+        }
         else if (isdigit(*c))
             while (isdigit(*++c));
 
@@ -254,7 +313,10 @@ int check_format_string(char *fmt)
         {
             precision = 1;
             if (*c == '*')
+            {
+                arg++;
                 c++;
+            }
             else
                 while (isdigit(*++c));
         }
@@ -279,8 +341,33 @@ int check_format_string(char *fmt)
                 break;
         }
 
-        if (*c == 's' && !precision)
-            return 1;
+        if (format_arg == -1)
+        {
+            if (*c == 's' && !precision)
+                return 1;
+        }
+        else
+        {
+            switch (*c)
+            {
+                case 's':
+                    if (!get_argument(arg)->is_constant)
+                        return 1;
+                    /* FALL THROUGH */
+
+                case 'c':
+                case 'd': case 'i':
+                case 'o': case 'u':
+                case 'x': case 'X':
+                case 'e': case 'E':
+                case 'f': case 'F':
+                case 'g': case 'G':
+                case 'a': case 'A':
+                case 'p': case 'n':
+                    arg++;
+                    break;
+            }
+        }
     }
 
     return 0;
@@ -342,10 +429,10 @@ int check_argument(int token, void *arg)
 
     if (data->current->is_constant)
     {
-        if (token != STRING_CONST)
+        if (token < HEX_CONST || token > STRING_CONST)
             data->current->is_constant = 0;
-        else if (!data->current->contains_ps)
-            data->current->contains_ps = check_format_string(yytext);
+        else if (token == STRING_CONST && !data->current->contains_ps)
+            data->current->contains_ps = check_format_string(*ratslexer.yytext, -1);
     }
 
     return 1;
@@ -377,13 +464,31 @@ argument_t *get_argument(int number)
     return arg;
 }
 
+
+static
+Vuln_t *locate_identifier(char *identifier)
+{
+
+    Vuln_t *data = NULL;
+    
+     
+    if (ratslexer.langhash) 
+        data = HashGet(ratslexer.langhash, identifier);
+    if ((data == NULL) && defaultdb)
+    {
+        data = HashGet(defaultdb, identifier);
+    }
+    return data;
+}
+
+
 static
 int push_identifier(char *identifier, int lineno)
 {
     Vuln_t *    data;
     stack_t *   frame;
 
-    if (database == NULL || (data = HashGet(database, identifier)) == NULL)
+    if (database == NULL || (data = locate_identifier(identifier)) == NULL)
         return 0;
 
     frame = (stack_t *)malloc(sizeof(stack_t));
@@ -446,8 +551,8 @@ void analyze_identifier(void)
     Vuln_t *        data;
     argument_t *    arg;
 
-    lineno     = lex_lineno;
-    identifier = strdup(yytext);
+    lineno     = *ratslexer.lex_lineno;
+    identifier = strdup(*ratslexer.yytext);
 
     if (!push_identifier(identifier, lineno))
     {
@@ -480,17 +585,26 @@ void analyze_identifier(void)
     {
         analyzed++;
         if ((arg = get_argument(data->FSProblem->Arg)) != (argument_t *)NULL)
+        {
             if (!arg->is_constant)
                 log_vulnerability(FSProblem, data->FSProblem->Severity);
+        }
     }
 
     if (data->BOProblem != (BOProblem_t *)NULL)
     {
         analyzed++;
         if (data->BOProblem->FormatArg > 0)
+        {
             if ((arg = get_argument(data->BOProblem->FormatArg)) != (argument_t *)NULL)
-                if (!arg->is_constant || arg->contains_ps)
+            {
+                if (!arg->is_constant)
                     log_vulnerability(BOProblem, data->BOProblem->Severity);
+            }
+            if (arg->is_constant && arg->contains_ps)
+                if (check_format_string(arg->yytext, data->BOProblem->FormatArg))
+                    log_vulnerability(BOProblem, data->BOProblem->Severity);
+        }
         if (data->BOProblem->SrcBufArg > 0)
             if ((arg = get_argument(data->BOProblem->SrcBufArg)) != (argument_t *)NULL)
                 if (!arg->is_constant)
@@ -589,14 +703,89 @@ void process_toctou(void)
     toctou_count = 0;
 }
 
-void process_file(char *filename, FILE *fd)
+
+void setup_perl(FILE *fd)
+{
+
+  yyperlin = fd;
+  ratslexer.yytext = &yyperltext;
+  ratslexer.yyin =  fd;
+  ratslexer.yylex = yyperllex;
+  ratslexer.yycomment = &yyperlcomment;
+  ratslexer.lex_lineno = &perllex_lineno;
+  ratslexer.langhash = (Hash)HashGet(database, "perl");
+  ratslexer.lang = LANG_PERL;
+}
+
+
+void setup_python(FILE *fd)
+{
+
+  yypin = fd;
+  ratslexer.yytext = &yyptext;
+  ratslexer.yyin =  fd;
+  ratslexer.yylex = yyplex;
+  ratslexer.yycomment = &yypcomment;
+  ratslexer.lex_lineno = &plex_lineno;
+  ratslexer.langhash = (Hash)HashGet(database, "python");
+  ratslexer.lang = LANG_PYTHON;
+}
+
+void setup_c(FILE *fd)
+{
+
+  yycin = fd;
+  ratslexer.yytext = &yyctext;
+  ratslexer.yyin = fd;
+  ratslexer.yylex = yyclex;
+  ratslexer.yycomment = &yyccomment;
+  ratslexer.lex_lineno = &clex_lineno;
+  ratslexer.langhash = (Hash)HashGet(database, "c");
+  ratslexer.lang = LANG_C;
+}
+
+
+
+void determine_language(char *filename, FILE *fd, int forcelang)
+{
+
+  char *dot;
+
+
+  dot = strrchr(filename, '.');
+  
+  if (forcelang)  {
+      if (forcelang == LANG_PYTHON)
+          setup_python(fd);
+      else if (forcelang == LANG_C)
+          setup_c(fd);
+      else if (forcelang = LANG_PERL)
+          setup_perl(fd);
+      return;
+  }
+  if (!dot)
+  {
+    setup_c(fd);
+    return;
+  }    
+  if (!strcmp(dot, ".py") || !strcmp(dot, ".PY"))  {
+    setup_python(fd);
+  } else if (!strcmp(dot, ".pl") || !strcmp(dot, ".pm"))  {
+    setup_perl(fd);
+  } else {
+    setup_c(fd);
+  }
+}
+
+void process_file(char *filename, FILE *fd, int forcelang)
 {
     int             i;
     accumulator_t * acc;
 
     /* (Re-)Initialize state */
-    yyin = fd;
-    lex_lineno = 1;
+    determine_language(filename, fd, forcelang);
+    *ratslexer.lex_lineno = 1;
+    last_text_line = 0;
     current_file = strdup(filename);
     for (i = 0;  i < DEPTH_COUNT;  depths[i++] = 0);
 
@@ -605,6 +794,7 @@ void process_file(char *filename, FILE *fd)
     process_toctou();
 
     /* Cleanup */
+    current_ignore = (ignore_t *)NULL;
     while ((acc = accumulators) != (accumulator_t *)NULL)
     {
         if (*(acc->text) != (char *)NULL)
@@ -612,3 +802,43 @@ void process_file(char *filename, FILE *fd)
         pop_accumulator();
     }
 }
+
+static
+void analyze_comment(void)
+{
+    ignore_t *  ign;
+    char *      c;
+
+    if (yyclength < 5)
+        return;
+
+    for (c = *ratslexer.yycomment;  *c && isspace(*c);  c++);
+    if (!*c)
+        return;
+    if (strncasecmp(c, "its4:", 5) && strncasecmp(c, "rats:", 5))
+        return;
+
+    for (c += 5;  *c && isspace(*c);  c++);
+    if (!*c)
+        return;
+    if (strncasecmp(c, "ignore", 6))
+        return;
+
+    for (c += 6;  *c && isspace(*c);  c++);
+    if (!*c || (*c != '$' && *c != '_' && !isalpha(*c)))
+    {
+        ign = new_ignore(*ratslexer.lex_lineno, (char *)NULL);
+    }
+    else
+    {
+        char *  p;
+
+        for (p = c;  *p && (isalnum(*p) || *p == '$' || *p == '_');  p++);
+        *p = '\0';
+        ign = new_ignore(*ratslexer.lex_lineno, c);
+    }
+
+    /* are we on a line that we've already encountered text on? */
+    if (last_text_line != *ratslexer.lex_lineno)
+        current_ignore = ign;
+}
