@@ -30,6 +30,25 @@
 #define INCLUDE_ALL_REFERENCES  0x1
 #define INPUT_MODE              0x2
 
+#define LANG_PYTHON		1	
+#define LANG_C		 	2	
+#define LANG_PERL		3
+
+
+typedef struct _lexer_t lexer_t;
+
+struct _lexer_t
+{
+    char **	    yytext;
+    FILE *	    yyin;
+    int 	    (*yylex)();
+    char **	    yycomment;
+    int *	    lex_lineno;
+    Hash	    langhash;
+    int		    lang;
+};
+
+    
 typedef struct _argument_t argument_t;
 struct _argument_t
 {
@@ -89,9 +108,10 @@ typedef int (* processorfn_t)(int, void *);
 
 extern int          flags;
 extern Hash         database;
+extern Hash	    defaultdb;
 extern char *       current_file;
 extern stack_t *    current_frame;
 
-extern void process_file(char *, FILE *);
+extern void process_file(char *, FILE *, int);
 
 #endif  /* ENGINE_H */
