@@ -25,10 +25,13 @@
 
 int warning_level = 2;
 
-static input_t *            input_head = (input_t *)NULL;
-static input_t *            input_tail = (input_t *)NULL;
-static vulnerability_t *    list_head  = (vulnerability_t *)NULL;
-static vulnerability_t *    list_tail  = (vulnerability_t *)NULL;
+static input_t *            input_head  = (input_t *)NULL;
+static input_t *            input_tail  = (input_t *)NULL;
+static ignore_t *           ignore_list = (ignore_t *)NULL;
+static vulnerability_t *    list_head   = (vulnerability_t *)NULL;
+static vulnerability_t *    list_tail   = (vulnerability_t *)NULL;
+
+static int lookup_ignore(char *filename, int lineno, char *token);
 
 static
 void insert_vulnerability(vulnerability_t *log)
@@ -147,6 +150,22 @@ void log_vulnerability(type_t type, Severity_t severity)
     insert_vulnerability(log);
 }
 
+void log_pythonbacktick(int lineno, Severity_t severity)
+{
+
+    vulnerability_t *   log;
+
+    log = (vulnerability_t *)malloc(sizeof(vulnerability_t));
+    log->filename = current_file;
+    log->lineno   = lineno;
+    log->data     = (Vuln_t *)NULL;
+    log->type     = PythonBacktick;
+    log->severity = severity;
+    log->uses     = (toctou_use_t *)NULL;
+
+    insert_vulnerability(log);
+
+}
 void log_staticbuffer(type_t type, int lineno, Severity_t severity)
 {
     vulnerability_t *   log;
@@ -300,6 +319,10 @@ void report_vulnerability(vulnerability_t *ptr)
             printf("pointer to function.\n\n");
             break;
 
+        case PythonBacktick:
+            printf("Beware of using backticks in python\n");
+            break;
+
         case None:
             printf("Unknown!?!?\n\n");
             break;
@@ -319,8 +342,11 @@ void report_inputs(void)
     for (ptr = input_head;  ptr != (input_t *)NULL;  ptr = next)
     {
         next = ptr->next;
-        count++;
-        printf("%s: %d: %s\n", ptr->filename, ptr->lineno, ptr->data->Name);
+        if (!lookup_ignore(ptr->filename, ptr->lineno, ptr->data->Name))
+        {
+            count++;
+            printf("%s: %d: %s\n", ptr->filename, ptr->lineno, ptr->data->Name);
+        }
         free(ptr);
     }
     input_head = input_tail = (input_t *)NULL;
@@ -336,11 +362,13 @@ void report_inputs(void)
 
 void generate_report(void)
 {
+    char *              lookup;
     char *              name;
     char *              name2 = (char *)NULL;
+    ignore_t *          iptr;
+    ignore_t *          inext;
     vulnerability_t *   ptr;
     vulnerability_t *   next;
-
     for (ptr = list_head;  ptr != (vulnerability_t *)NULL;  ptr = ptr->next)
     {
         if (ptr->severity == Default || ptr->severity >= warning_level)
@@ -353,27 +381,40 @@ void generate_report(void)
                 case InputProblem:
                 case RaceConditionCheck:
                 case RaceConditionUse:
-                    name = ptr->data->Name;
+                    name = lookup = ptr->data->Name;
                     break;
 
                 case StaticLocalBuffer:
                     name = "fixed size local buffer";
+                    lookup = "$fixed_buffer$";
                     break;
 
                 case StaticGlobalBuffer:
                     name = "fixed size global buffer";
+                    lookup = "$global_buffer$";
                     break;
 
                 case Reference:
                     name = "non-function call reference";
-                    name2 = ptr->data->Name;
+                    name2 = lookup = ptr->data->Name;
+                    break;
+               
+                case PythonBacktick: 
+                    name = "backtick";
+                    lookup = "$python_backtick$";
                     break;
 
                 case None:
                 default:
                     name = "Unknown / Database Error";
+                    lookup = (char *)NULL;
                     break;
             }
+            if (lookup != (char *)NULL)
+            {
+                if (lookup_ignore(ptr->filename, ptr->lineno, lookup))
+                    continue;
+            }
             if (name2 == (char *)NULL)
                 printf("%s:%d: %s: %s\n", ptr->filename, ptr->lineno, severities[ptr->severity], name);
             else
@@ -388,6 +429,15 @@ void generate_report(void)
 
     report_inputs();
 
+    for (iptr = ignore_list;  iptr != (ignore_t *)NULL;  iptr = inext)
+    {
+        inext = iptr->next;
+        if (iptr->token != (char *)NULL)
+            free(iptr->token);
+        free(iptr);
+    }
+    ignore_list = (ignore_t *)NULL;
+
     for (ptr = list_head;  ptr != (vulnerability_t *)NULL;  ptr = next)
     {
         next = ptr->next;
@@ -395,3 +445,36 @@ void generate_report(void)
     }
     list_head = list_tail = (vulnerability_t *)NULL;
 }
+
+ignore_t *new_ignore(int lineno, char *token)
+{
+    ignore_t *  ign;
+
+    if ((ign = (ignore_t *)malloc(sizeof(ignore_t))) == (ignore_t *)NULL)
+        return (ignore_t *)NULL;
+    ign->filename = current_file;
+    ign->lineno   = lineno;
+    ign->token    = (token == (char *)NULL ? token : strdup(token));
+    ign->next     = ignore_list;
+    ignore_list   = ign;
+
+    return ign;
+}
+
+static
+int lookup_ignore(char *filename, int lineno, char *token)
+{
+    ignore_t *  ptr;
+
+    for (ptr = ignore_list;  ptr != (ignore_t *)NULL;  ptr = ptr->next)
+    {
+        if (ptr->filename != filename)  /* yes, this is safe and will work */
+            continue;
+        if (ptr->lineno != lineno)
+            continue;
+        if (ptr->token == (char *)NULL || !strcmp(ptr->token, token))
+            return 1;
+    }
+
+    return 0;
+}
